<?php
// $Id: regcode.admin.php,v 1.1.2.3 2009/06/17 09:35:54 aidan Exp $

require_once('regcode.api.php');

/**
 * Return the form associated with the module settings.
 *
 * @return
 *   The settings form.
 */
function regcode_admin_settings() {
  $form = array();
  
  $form['regcode_optional'] = array(
    '#type' => 'checkbox',
    '#title' => t("Make registration code optional"),
    '#default_value' => variable_get('regcode_optional', 0),
    '#description' => t('If checked, users can register without a registration code. ') .
                      t('Alternatively, there is also a permission to allow certain roles (e.g. admins) to bypass´registration code entry for administratively added users.'),
  );
  
  $form['regcode_fieldset_title'] = array(
    '#type' => 'textfield',
    '#title' => t("Field title"),
    '#description' => t('The title of the registration code textfield'),
    '#default_value' => variable_get('regcode_fieldset_title', 'Registration Code'),
  );

  $form['regcode_field_title'] = array(
    '#type' => 'textfield',
    '#title' => t("Field title"),
    '#description' => t('The title of the registration code textfield'),
    '#default_value' => variable_get('regcode_field_title', 'Registration Code'),
  );

  $form['regcode_field_description'] = array(
    '#type' => 'textfield',
    '#title' => t("Field description"),
    '#description' => t('The description of the registration code textfield'),
    '#default_value' => variable_get('regcode_field_description', 'Please enter your registration code.'),
  );

  return system_settings_form($form);
}


/**
 * Create a registration code
 *
 * @return
 *   The settings form.
 */
function regcode_admin_create() {
  $form = array();
  
  $form['regcode_create_code'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Registration code"),
    '#description'   => t('The registration code as the user would enter it. Leave blank to have code generated.'),
  );

  $form['regcode_create_category'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Assign a category"),
    '#size'          => 30,
    '#autocomplete_path' => 'admin/user/regcodes/autocomplete',
    '#description'   => t('Optional classification category for code management'),
  );
  
  $form['regcode_create_maxuses'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Maximum number of uses"),
    '#default_value' => 1,
    '#size'          => 10,
    '#description'   => t('Whether this code can only be used once, or multiple times'),
  );

  $form['regcode_create_begins'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Begins"),
    '#description'   => t('When this code should activate'),
  );
  
  $form['regcode_create_expires'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Expires"),
    '#description'   => t('When this code should expire'),
  );

  $form['regcode_create_active'] = array(
    '#type'          => 'checkbox',
    '#title'         => t("Enable code"),
    '#default_value' => 1,
    '#description'   => t('Whether this code can be used immediately'),
  );
  
  $form['regcode_generate'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("Bulk code generation"),
    '#description'   => t('These options are ignored if a registration code is specified'),
  );
  
  $form['regcode_generate']['regcode_generate_number'] = array(
    '#type'          => 'textfield',
    '#default_value' => 1,
    '#size'          => 10,
    '#title'         => t('Number of codes'),
  );
  
  $form['regcode_generate']['regcode_generate_length'] = array(
    '#type'          => 'textfield',
    '#default_value' => variable_get('regcode_generate_length', '10'),
    '#size'          => 10,
    '#title'         => t('Length of codes'),
    '#description'   => t('How many characters should be used in the generated codes'),
  );
  
  $form['regcode_generate']['regcode_generate_format'] = array(
    '#type'          => 'select',
    '#default_value' => variable_get('regcode_generate_format', 'alpha'),
    '#title'         => t('Format of the generated codes'),
    '#options'       => array('alpha' => 'Letters', 'numeric' => 'Numbers', 'alphanum' => 'Numbers & Letters', 'hexadec' => 'Hexadecimal'),
  );
  
  $form['regcode_generate']['regcode_generate_case'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Uppercase generated codes'),
    '#default_value' => variable_get('regcode_generate_case', ''),
  );
  
  $form['regcode_create_submit'] = array(
    '#type'          => 'submit',
    '#value'         => t("Create"),
  );
  
  return $form;
}


/**
 * Validator for create form
 */
function regcode_admin_create_validate($form, &$form_state) {
  if (!empty($form_state['values']['regcode_create_begins']) &&
      strtotime($form_state['values']['regcode_create_begins']) < 1) {

    form_set_error('regcode_create_code', 'Invalid begin date');      
  }
  
  if (!empty($form_state['values']['regcode_create_expires']) &&
      strtotime($form_state['values']['regcode_create_expires']) < 1) {

    form_set_error('regcode_create_code', 'Invalid expiry date');      
  }
}


/**
 * Submit handler for regcode_admin_create
 */
function regcode_admin_create_submit($form, &$form_state) {
  $code = array();
  if (!empty($form_state['values']['regcode_create_begins'])) {
    $code['begins'] = date('Y-m-d h:j:s', strtotime($form_state['values']['regcode_create_begins']));
  }
  
  if (!empty($form_state['values']['regcode_create_expires'])) {
    $code['expires'] = date('Y-m-d h:j:s', strtotime($form_state['values']['regcode_create_expires']));
  }
  
  $code['category']   = $form_state['values']['regcode_create_category'];
  $code['is_active']  = $form_state['values']['regcode_create_active'];
  $code['maxuses']    = $form_state['values']['regcode_create_maxuses'];
  $code['code']       = $form_state['values']['regcode_create_code'];
  
  for ($i = 0; $i < $form_state['values']['regcode_generate_number']; $i++) {
    if ($form_state['values']['regcode_generate_number'] > 1 ||
        empty($form_state['values']['regcode_create_code'])) {
        
      $code['code'] = regcode_generate_code($form_state['values']['regcode_generate_length'],
        $form_state['values']['regcode_generate_format'], 
        $form_state['values']['regcode_generate_case']);
    }
    
    if (regcode_save_code($code, 'skip')) {
      drupal_set_message(t('Created registration code (@code)', array('@code' => $code['code'])));
    } else {
      drupal_set_message(t('Unable to create code (@code) as code already exists', array('@code' => $code['code'])), 'warning');  
    }
  }
}


/**
 * Return the code list page content with(in) the according filter form
 *
 * @return
 *   The settings form.
 */
function regcode_admin_list() {

  $form = array();

  $form['#method'] = 'get';

  $codes = regcode_get_codes();
  $keys = array('ID', 'UID', 'Created', 'Lastused', 'Begins', 'Expires', 'Category', 'Code', 'Active', 'Max uses', 'Uses');
  $output = t('No codes found');
  if (!empty($codes)) {
    $output = theme('table', $keys, $codes);
  }
  
  $form['list'] = array(
      '#type'  => 'markup',
      '#value' => $output,
  );

  return $form;
}


/**
 * Return the form associated with the registration code import admin page
 *
 * @return
 *   The import form array
 */
function regcode_admin_import() {

  $form['#cache'] = FALSE;

  $form['#attributes']['enctype'] = 'multipart/form-data';

  $form['regcode_import_action'] = array(
      '#type' => 'select',
      '#title' => t("Action"),
      '#default_value' => variable_get('regcode_import_action', 'skip'),
      '#options' => array(
                          'skip'      => t('Skip'),
                          'overwrite' => t('Overwrite'),
                          'clean'     => t('Clean'),
                          ),
      '#description' => t('Action to perform when importing the data.<br />
                           - <strong>Skip</strong>: Add new codes skipping those which already exists.<br />
                           - <strong>Overwrite</strong>: Add new codes overwriting those which already exists.<br />
                           - <strong>Clean</strong>: Erases *all* existing codes before importing new codes.<br />
                          '),
  );

  if ($form['regcode_import_action']['#default_value'] == 'clean') {
    $form['regcode_import_action_check'] = array(
        '#type' => 'checkbox',
        '#title' => t("Really CLEAN (wipe out) all existing codes with the selected import action?"),
        '#default_value' => 0,
        '#required' => TRUE,
        '#validate' => array('regcode_import_action_check_validation' => array()),
    );    
  }

  $form['regcode_import_file'] = array(
      '#type' => 'file',
      '#title' => t("File"),
      '#description' => t('File upload with data to add/import (plaintext list or CSV format)'),
  );

  $form['regcode_import_text'] = array(
      '#type' => 'textarea',
      '#cols' => 80,
      '#rows' => 10,
      '#title' => t("Data"),
      '#description' => t('Direct text input with data to add/import (plaintext list or CSV format)'),
  );
  

  $form['regcode_import'] = array(
    '#type'                => 'fieldset',
    '#title'               => t("Import Settings"),
    '#description'         => t('Settings to use for code import.'),
    '#collapsible'         => TRUE,
    '#collapsed'           => TRUE,
  );

  $form['regcode_import']['regcode_import_csv_delimiter'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Field delimiter"),
    '#description'   => t('Character used as delimiter in csv import of registration codes'),
    '#default_value' => variable_get('regcode_import_csv_delimiter', ','),
  );

  $form['regcode_import']['regcode_import_csv_text_enclosure'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Text enclosure"),
    '#default_value' => variable_get('regcode_import_csv_text_enclosure', '"'),
    '#description'   => t('Character used as enclosure around text content fields on CSV import.'),
  );
  
  $form['regcode_import']['regcode_import_csv_fieldorder'] = array(
    '#type'          => 'textfield',
    '#title'         => t("Field order"),
    '#default_value' => variable_get('regcode_import_csv_fieldorder', 'code'),
    '#description'   => t('Order in which fields are expected in the CSV file (field names separated by delimiter set above).<br />
                           Leave BLANK to use a field header line in the CSV data.<br />
                           Example: "IGNORE,code,rid,info"<br />
                           Available field names:
                           <pre>
  IGNORE      - Ignore the data at this field position
  code        - The code string itself
  begins      - Date string to activate registration code
  expires     - Date string to deactivate registration code
  category    - Category to assign this code to
  code        - The registration code
  is_active   - Activate this code immediately
  is_onetime  - Allow this code to be used once only
                           </pre>'),
  );

  $form['submit'] = array(
    '#type'  => 'submit',
    '#title' => t('Import'),
    '#value' => 'submit',
  );

  return $form;
}


/**
 * Handle the processing of a submitted import form
 *
 */
function regcode_admin_import_submit($form_id, $form_values) {

  // assemble code template
  foreach (regcode_get_fields(FALSE, TRUE) as $field_name => $field_title) {
    $field_name  = str_replace('c.', '', $field_name);
    $field_input = check_plain(trim($form_values['regcode_template_'. $field_name]));
    if (trim($field_input) != '') $code_template[$field_name] = $field_input;
  }

  // determine whether to import from file ...
  $import_file = file_check_upload('regcode_import_file');
  if (is_object($import_file)) {
    $result = regcode_import_file($import_file->filepath, $form_values['regcode_import_action'], $code_template);
    unlink($import_file->filepath);
  }
  // ... or from text area
  else {
    $result = regcode_import_text($form_values['regcode_import_text'], $form_values['regcode_import_action'], $code_template);
  }

  if ($result) {
    drupal_set_message(t('Registration code import successful'));
  }
  else {
    drupal_set_message(t('Registration code import failed!'), 'error');
  }
}


/**
 * Validates the regcode_import_action_check checkbox
 *
 * @param element
 *  The element of the checkbox
 */
function regcode_admin_import_validate() { 
  if (empty($element['#value'])) {
    form_error($element, t('!title field is required.', array('!title' => filter_xss_admin(($element['#title'])))));
  }
  
  return;
}


/**
 * Autocomplete helper for categories
 */
function regcode_admin_categories($string) {
  $matches = array();
  if (!empty($string)) {
    $result = db_query_range("SELECT DISTINCT category FROM {regcode} WHERE LOWER(category) LIKE LOWER('%s%%')", $string, 0, 10);
    while ($regcode = db_fetch_object($result)) {
      $matches[$regcode->category] = check_plain($regcode->category);
    }
  }
  
  print drupal_to_js($matches);
}


/**
 *
 */
function regcode_admin_export() {
  return 'not implemented';
}
